# -*- coding: utf-8 -*-
"""
Created on Fri Aug 16 14:29:24 500019

@author: Nitsch
"""


import os
import numpy as np
import matplotlib.pyplot as plt
import pandas as pandas
from matplotlib import cm
# import matplotlib.animation as animation
from matplotlib.animation import FuncAnimation
# from matplotlib.animation import FuncAnimation
# from matplotlib.widgets import Slider, Button
# import matplotlib.ticker as mtick

###################### Costumisation


# Ticks
plt.rcParams['xtick.direction'] = 'in'
plt.rcParams['ytick.direction'] = 'in'

# Fonts
families = ['serif', 'sans-serif', 'cursive', 'fantasy', 'monospace']
weights = ['light', 'normal', 'medium', 'semibold', 'bold', 'heavy', 'black']
plt.rcParams['font.size'] = 12
plt.rcParams['font.family'] = families[0]      # Set the font family, available are: families = ['serif', 'sans-serif', 'cursive', 'fantasy', 'monospace']
plt.rcParams['font.weight'] = weights[1]       # Set the fonts weights, available are: ['light', 'normal', 'medium', 'semibold', 'bold', 'heavy', 'black']
plt.rcParams['legend.fontsize'] = 10
plt.rcParams['legend.facecolor'] = 'white'
plt.rcParams['legend.framealpha'] = None  
plt.rcParams['legend.title_fontsize'] =10    # None sets to the same as the default axes.

plt.rcParams['grid.linewidth'] = 0.3        # Set the grid color
plt.rcParams['grid.color'] = 'silver'          # Set the grid color
plt.rcParams["font.family"] = "DejaVu Sans"

                        
# Color schemes generated with "internal" color maps          
start = 0.0
stop = 1.0

#%%

# Data Thebes
datath = pandas.read_csv("data.dat", skipinitialspace=False, delim_whitespace=True, skiprows=0)

#%%
# enter the name of your directory here
name = 'infil'#'-intel-kng1d6_inc_k'#'-days'

curdir = os.getcwd()
outdir = os.path.join(curdir,name)

fig, axe = plt.subplots(nrows=1, ncols=1,figsize=(6,4.5))
# plt.title('numgeo: ' + os.path.basename(outdir))

    
counter = 0

if os.path.exists(outdir+'\\mesh_Soil.inp') == True:
    print('mesh_soil')
    # mesh_Soil
    names = [67,333,328,323,318,313,308,303,298,293,288,283,278,273,268, \
          263,258,253,248,243,238,233,228,223,218,213,208,203,198,193,131]
    
elif os.path.exists(outdir+'\\mesh_Soil_fine.inp') == True:
    print('mesh_soil_fine')
    # mesh_soil_fine      
    names = [89,454,449,444,439,434,429,424,419,414,409,404,399,394,389,384,\
             379,374,369,364,359,354,349,344,339,334,329,324,319,314,309,304,\
                 299,294,289,284,279,274,269,264,259,175]
   
else:
    print('Error: different mesh used')
 

number_of_lines = len(names)+10
Reds = [ cm.Reds(x) for x in np.linspace(start, stop, number_of_lines)  ] ; Reds = Reds[::-1]


non= len(names)
      
for n in np.arange(1,non+1,1).tolist():
    df = pandas.read_csv(outdir+"\\infil-print-out\\NOUT_node_"+str(names[n-1])+".dat", skipinitialspace=False, delim_whitespace=True, skiprows=0)
    
    por = df['Pw']#-df['Pw'][0]
    t = df['Time']
    
    #------------------------------------------------------------------------
    # create an array, where data from all points is in one row for one time
    if counter == 0:
        newdf = df[['Time', 'Pw']].copy()
        newdf.columns = ['Time', 'Pw_'+str(names[n-1])]
    else:
        newdf.insert(counter+1,'Pw_'+str(names[n-1]),por)

    
    tmp=newdf.to_numpy()
    if counter == 0:
        tmp[:,0] = tmp[:,0]/(60*60*24)
    # x = tmp[]
        
    
    
    # get y-coordinates of nout
    coords = df['X2'][0]
    if counter == 0:
        y = np.zeros(non)
        y[n-1] = coords
    else:
        y[n-1] = coords
     

    counter += 1


# define the interval for which times the results should be evaluated
# num=15
# rg = np.arange(0,len(tmp),num)
# for m/s input with inc_max=500
rg=[0,66,100,135,221,567,1777]


# color map
number_of_lines2 = len(rg)+10
Blues = [ cm.Blues(x) for x in np.linspace(start, stop, number_of_lines2)  ] ; Blues = Blues[::-1]

t_in = np.array([])
ct = 0
for i in rg:
    axe.plot(tmp[i,1:]*(-1),y,'.k-',lw=0.8,color=Blues[ct],label='t=%5.3f d' %tmp[i,0])
    # define input t_in for analytical solution
    t_in=np.append(t_in,tmp[i,0])
    ct +=1


#%%
# Analytical solution from Srivastava and Yeh, 1991
 
# Roots from MMA       
sol= np.loadtxt("sol.txt")

# constants
# enter here
zeta = 5.0
q = 0.005 # m/day
# 0.005 m/day = 5.787d-8 m/s
K = 0.1 # m/day
# 0.1 m/day = 1.157d-6 m/s = 1.157d-13 m^2
l= 1.0
n = 0.4
Ssat = 1.0
Sres = 0.23

 
# evaluates the sum over list sol from MMA   
def func(sol,z,t):
    zstar = z*zeta
    lstar = zeta * l
    tstar = zeta * K * t/(n*(Ssat-Sres))
    return np.sin(sol*zstar)*np.sin(sol*lstar)*np.exp(-sol**2*tstar)/(1+lstar/2+2*sol**2*lstar)

def psi(z,t,Sum):
    zstar = z*zeta
    lstar = zeta * l
    tstar = zeta * K * t/(n*(Ssat-Sres))
    # res = np.zeros(len(sol))
    B=  q/K-(q/K-1)*np.exp(-zstar)-4*q/K*np.exp((lstar-zstar)/2)*np.exp(-tstar/4)*Sum
    return -1/zeta * np.log(B)    


# take depths for analytical solution from numgeo calculation
# times are already stored in t_in above
z_in = y[::-1]

# allocate arrays for solution
res = np.zeros(len(sol))

x = np.zeros([len(z_in),len(t_in)])


# loop over depth
for i in np.arange(0,len(z_in),1):
    # loop over time
    for j in np.arange(0,len(t_in),1):
        # loop over entries in sol from MMA
        for k in np.arange(0,len(sol),1):
            # res = np.zeros(len(sol))
            # evaluate the term inside the summation
            res[k] = func(sol[k],z_in[i],t_in[j])
            # print(res)
        # calculate the sum of res    
        Sum=sum([i for i in res])
        # print(Sum)
        # Sum = 0.2483
        # Evaulate the result for psi
        x[i,j] = psi(z_in[i],t_in[j],Sum)
        # print(x)   


# -----------------------
# Analytical solution implemented in MMA/Python
number_of_lines2 = len(t_in)+5
Oranges = [ cm.Oranges(x) for x in np.linspace(start, stop, number_of_lines2)  ] ; Oranges = Oranges[::-1]
for j in np.arange(0,len(t_in),1):
    plt.plot(x[:,j]*10, z_in, marker='x',color=Oranges[j],lw='0.8', ls='--',markersize='4', label='t= %5.3f d' %t_in[j])
# -----------------------   

axe.annotate('numgeo', xy=(6, 0.4), xytext=(6,0.35), fontsize='10')
# axe.annotate("", xytext=(220, 120), xy=(280,240), arrowprops=dict(arrowstyle="<|-|>",fc='k',lw=0.8))

axe.annotate('Analytical \nsolution', xy=(8.5, 0.4), xytext=(8.5,0.35), fontsize='10')

axe.set_xlabel('Pore water pressure / kPa')
axe.set_ylabel('Height / m')
leg=axe.legend(loc="lower right",fontsize=8,ncol=2)
leg.get_frame().set_linewidth(0.5)
axe.grid(True,'both')

plt.tight_layout(w_pad=0.9)
    

            






